//------------------------------------------------------------------------
void setupK4s() {
  // set all K4 pins to OUTPUT
  for (int i = 0; i < 8; i++) {
    pinMode(pressureSprayer[i], OUTPUT);
  }
}

/*global arrays in the head of main file
  boolean       K4state[8];             //stores if channel is HIGH or LOW
  int           K4on[8];                //time in MS how long channel should be on
  int           K4off[8];               //time in MS how long channel should be off
  int           K4repeat[8];            //number of repetitions per on/off cycle
  unsigned long K4time[8];              //saves time in millis of last LOW->HIGH flange
*/

// -------------------------------------------- triggerK4() initiates a pwm controlled spray on one channel

void triggerK4(int index, int on, int off, int repeat) {
  // constrain incoming values
  if (on < 0) {
    on = 0;
  }
  if (off < 0) {
    off = 0;
  }
  if (repeat < 0) {
    repeat = 0;
  }
  K4on[index] = on;              //time in MS how long channel should be on
  K4off[index] = off;            //time in MS how long channel should be off
  K4repeat[index] = repeat;      //number of repetitions per on/off cycle
  K4time[index] = millis();      //saves time in millis of last LOW->HIGH flange
}

// -------------------------------------------- updateK4s() drives all K4 channels - should be called once per main loop
void updateK4s() {

  for (int i = 0; i < maxK4; i++) {             //cycle through all preasure sprayer channels
    
      if (K4repeat[i] > 0) {                    //check if output is in an active loop
        unsigned long currentTime = millis();

        if (K4time[i] + K4on[i] > currentTime ) { //is the longest time the channel should be turned on later than the current time? (=should this channel be on?)
          if (K4state[i] == LOW) {              //is the channel turned off?
            K4state[i] = HIGH;                  //-> turn channel on
            K4time[i] = millis();               //-> remember when this channel was turned on
          }
        }

        if (K4time[i] + K4on[i] <= currentTime ) {  //is the longest time the channel should be turned on earlier than the current time? (=should this channel be off?)
          if (K4state[i] == HIGH) {                //is the channel turned on?
            K4state[i] = LOW;                   //-> turn channel off
            K4repeat[i]--;                      //-> decrement repeat index
          }
        }

        if (K4time[i] + K4on[i] + K4off[i] < currentTime ) { //is the longest time the channel should be turned on and off earlier than the current time? (=should this channel be turned on again?)
          if (K4state[i] == LOW && K4repeat > 0) { //is the channel turned off and should it repeat?
            K4state[i] = HIGH;                  //-> turn channel on
            K4time[i] = millis();               //-> remember when this channel was turned on
          }
        }
      }
      if (K4repeat[i] == 0) {
        K4state[i] = LOW;
      }
      digitalWrite(pressureSprayer[i], K4state[i] && (!safetyOff)); //update output channels, set K4 outputs to low if there's a storm detected
      if (VERBOSE_FOUNTAINS) {
        if (i == 5) {
          Serial.print("   ");
        }
        if (K4state[i]) {
          Serial.print("|");
        } else {
          Serial.print(".");
        }
      }
   
  }
  if (VERBOSE_FOUNTAINS)Serial.println();
}


