
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void interpretePiano() {
  /* electric piano pressure sprayer interpreter:
     the sprays in the pool are animated and symmetrical
     every push of a key will move the spray (agent) to a new target position
     while moving, the spray pulses are short - when arrived at target, spray pulse is a bit longer
     special effects (top sprayers) are used for chords:
     3 keys at the same time: sprayer on existing fountain kicks in
     4 keys at the same time and top down sprayer kicks in
     5 keys at the same time and roof sprayer kicks in
     all top sprayers only act once as long as any key remains pressed
     all top sprayers only work once every 5 seconds
  */
  int keyCount = 0;                                                   // used to count how many keys are pushed simultaneously (chord detection)
  boolean keyPushed = false;                                          // remembers if a key is newly pushed
  float pushedPositions = 0;                                          // used to calculate average key position on keyboard
  for (int i = 0; i < MAX_INPUT_KEYS; i++) {                          // cycle through all notes
    sprayer[i] = synth[i];                                            // copy synth[]-array to sprayer[]-array
    if (sprayer[i]) {
      keyCount++;                                                     // count keys that are pushed
      pushedPositions += i;                                           // add position of key
    }

    if (sprayer[i] && !pSprayer[i]) {                                 // if key was pushed
      keyPushed = true;                                               // a key was pushed

      agent1target += (i % 12) * 2 + 1;                               // move target to new position based on key
      agent1target = agent1target % 15;                               // cycle target position in order to avoid high numbers
      if (agent1pos == agent1target) {                                // if target matches position
        agent1target++;                                               // -> shift target a bit (every key press results in a visible outcome)
      }
    }
    pSprayer[i] = sprayer[i];                                         // remember interpreter input state
  }

  float averageKey = -1;                                              // calculate average keyposition of currently pushed keys on keyboard
  if (keyCount > 0) {                                                 // 0 = leftmost key ; 255.0 = rightmost key
    averageKey = pushedPositions / keyCount;
    averageKey = averageKey / (MAX_INPUT_KEYS - 1);
    averageKey = averageKey * 255;
  }

  //                                                                  //////////////////////////////LED interpretation
  if (keyPushed) {                                                    // if a key is freshly pushed
    int x = int(averageKey * .9);
    int r = gamma[sine[(x / 2 + 40) % 256]];
    int g = gamma[sine[(x / 2 + 64 + 180) % 256]];
    int b = gamma[sine[(x / 2 + 64 + 128) % 256]];
    alternateLED = !alternateLED;                                   // toggle which LED's should be addressed
    if (alternateLED) {
      fadeLEDs(0, 2000, r, g, b , r/8,g/8,b/8);     // fade 12V LED's
    } else {
      fadeLEDs(1, 2000, r, g, b , r/8,g/8,b/8);     // fade 12V LED's
    }
    

  }
  //                                                                  //////////////////////////////Pressure Sprayer interpretation
  if (keyCount > 0) {                                                 // if one or more keys are currently pushed:
    int agent1delay = 100;                                             // define how fast the agent should move from one sprayer to the next in ms
    if (agent1target != agent1pos && agent1time + agent1delay < millis()) { // check if agent has to move (position off && is it time?)
      agent1time = millis();                                          // remember when agent was moved
      if (agent1pos < agent1target) {                                 // move agent 1 step at a time towards target
        agent1pos++;
      } else {
        agent1pos--;
      }
      int sprayer = agent1pos % 5;                                    // select which sprayer(s) in pool should be addressed
      if (agent1pos == agent1target) {                                // is this the final position of the agent?
        triggerK4(sprayer, 700, 1800, 1);                                // -> turn sprayer on long
        triggerK4(4 - sprayer, 700, 1800, 1);                         // -> mirror: turn sprayer on long
      } else {                                                        // not the final position yet:
        triggerK4(sprayer, 300, 1200, 1);                                // -> turn sprayer on short
        triggerK4(4 - sprayer, 300, 1200, 1);                         // -> mirror: turn sprayer on long
      }
    }
  } else {                                                            // if no key is currently pushed:
    for (int i = 0; i < 5; i++) {                                     // -> kill all sprayers
      triggerK4(i, 0, 0, 0);
    }
    pianoK4FXreset = true;                                            // keys have been released -> special FX sprayers can be used again
  }
  //................................................................ special sprayers on top react on chords of at least 3 keys but need to be interrupted by no keys pushed:

  if (pianoK4FXreset == true && keyPushed) {                         // is it allowed to use special FX sprayers? ()

    if (keyCount == 3) {                                             // if 3 keys are pressed right now
      if (millis() > sprayer5lastOn + 1000) {                       // if sprayer #5 has not been used for 1 second
        triggerK4(5, 1500, 800, 1);                                    // initiate spray on existing fountain
        //sprayer5lastOn = millis();                                   // remember when sprayer #5 was used
        pianoK4FXreset = false;                                      // remember that FX were used (will be reseted when no keys are pushed)
      }
    }
    if (keyCount == 4) {                                             // if 3 keys are pressed right now
      if (millis() > sprayer7lastOn + 1000) {                       // if sprayer #7 has not been used for 1 second
        triggerK4(7, 1000, 700, 1);                                    // initiate spray going down
        sprayer7lastOn = millis();                                   // remember when sprayer #7 was used
        pianoK4FXreset = false;                                      // remember that FX were used (will be reseted when no keys are pushed)
      }
    }
    if (keyCount > 4 ) {                                             // if more than 4 keys are pressed right now
      if (millis() > sprayer6lastOn + 1000) {                       // if sprayer #6 has not been used for 1 second
        triggerK4(6, 3000, 700, 1);                                    // initiate spray on roof
        sprayer6lastOn = millis();                                   // remember when sprayer #6 was used
        pianoK4FXreset = false;                                      // remember that FX were used (will be reseted when no keys are pushed)
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



void interpreteSpaceSynth() {

  /* space synth pressure sprayer interpretation:
    the sprays in the pool are animated
    every push of a key will move the spray (agent) to a new target position
    while moving, the spray pulses are short - when arrived at target, spray pulse is a bit longer
    special effects occur when the agent travels more than 4 steps:
    channel 5,6&7 are triggered based on travelled steps % 3
    channel 5&6 can only be turned on from time to time
  */
  int keyCount = 0;                                                   // used to count how many keys are pushed simultaneously (chord detection)
  for (int i = 0; i < MAX_INPUT_KEYS; i++) {                          // cycle through all notes
    sprayer[i] = synth[i];                                            // copy synth[]-array to sprayer[]-array
    keyCount += sprayer[i];                                           // count how many keys are currently pushed
    if (sprayer[i] && !pSprayer[i]) {                                 // if key was pushed

      if (millis() > lastFade + 100) {
        lastFade = millis();                                          // save time when last LED was faded in order to avoid flashes
        switch (i % 3) {
          case 0:
            fadeLEDs(0, 1200,  255, 128, 64,    32, 255, 128);          // fade 12V LED's
            fadeLEDs(1, 650,  128, 64, 255,     255, 128, 32);
            break;
          case 1:
            fadeLEDs(0, 650,  128, 64, 255,    255, 128, 32);           // fade 12V LED's
            fadeLEDs(1, 1200,  255, 128, 64,    32, 255, 128);
            break;
          case 2:
            fadeLEDs(0, 650,  255, 64, 128,     64, 164, 255);           // fade 12V LED's
            fadeLEDs(1, 1200,  64, 128, 255,    32, 255, 128);
        }
      }

      agent1target += (i % 12) * 2 + 1;                               // move target to new position based on key
      agent1target = agent1target % 15;                               // cycle target position in order to avoid high numbers
      if (agent1pos == agent1target) {                                // if target matches position
        agent1target++;                                               // -> shift target a bit (every key press results in a visible outcome)
      }
    }
    pSprayer[i] = sprayer[i];                                         // remember interpreter input state
  }

  if (keyCount > 0) {                                                 // if one or more keys are currently pushed:
    int agent1delay = 150;                                             // define how fast the agent should move from one sprayer to the next in ms
    if (agent1target != agent1pos && agent1time + agent1delay < millis()) { // check if agent has to move (position off && is it time?)
      agent1time = millis();                                          // remember when agent was moved
      if (agent1pos < agent1target) {                                 // move agent 1 step at a time towards target
        agent1pos++;
      } else {
        agent1pos--;
      }
      agent1Travel++;                                                 // count the travelled step
      int sprayer = agent1pos % 5;                                    // select which sprayer(s) in pool should be addressed
      if (agent1pos == agent1target) {                                // is this the final position of the agent?
        triggerK4(sprayer, 500, 2000, 1);                               // -> turn sprayer on long
        if (agent1Travel > 4) {                                       // if agent travelled more than four steps:
          /////////////////////////////// Special FX! ///////////

          switch (agent1Travel % 3) {
            case 0:
              //if (millis() > sprayer5lastOn + 25000) {                        // if sprayer #5 has not been used for 25 seconds
                triggerK4(5, 500, 700, 1);                                  // initiate spray on 5
                //sprayer5lastOn = millis();                                    // remember when sprayer #5 was used
              //}
              break;
            case 1:
              //if (millis() > sprayer6lastOn + 25000) {                        // if sprayer #6 has not been used for 25 seconds
                triggerK4(6, 700, 600, 3);                                  // initiate spray on 6
                //sprayer6lastOn = millis();                                    // remember when sprayer #6 was used
              //}
              break;
            case 2:
              triggerK4(7, 300, 1500, 1);                                    // initiate spray on 7
              //sprayer7lastOn = millis();                                      // remember when sprayer #7 was used
              break;
          }
          /////////////////////////////// End of Special FX! ///////////
        }
      } else {                                                        // not the final position yet:
        triggerK4(sprayer, 200, 2500, 1);                                // -> turn sprayer on short
      }
    }
  } else {                                                            // if no key is currently pushed:
    agent1Travel = 0;                                                 // reset the agent step travel counter
    for (int i = 0; i < 5; i++) {                                     // -> kill all sprayers
      triggerK4(i, 0, 0, 0);
    }
  }
}





////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void interpreteGlockenspiel() {
  /* glockenspiel pressure sprayer interpretation:
     leftmost key will trigger sprayer on existing fountain 
     rigthmost key will trigger sprayer on roof  
     chord (3 or more keys) will additionally trigger sprayer shooting down 
     white keys will trigger sprayers in pool from left to right
     black keys will trigger sprayers in pool from right to left
  */
  int keyCount = 0;                                                   // used to count how many keys are pushed simultaneously (chord detection)
  int keyPushed = -1;                                                 // stores which key was pushed for LED interpretation
  for (int i = 0; i < MAX_INPUT_KEYS; i++) {                          // cycle through all notes
    sprayer[i] = synth[i];                                            // copy synth[]-array to sprayer[]-array
    keyCount += synth[i];                                             // count how many keys are pushed in order to detect chords
    if (sprayer[i] && !pSprayer[i]) {                                 // if sprayer[] was just turned on
      keyPushed = i;                                                  // remember the key that was pushed
      boolean lrKey = false;                                          // variable saves if utmost left or right key was pressed
      if (i == 0) {                                                   // if leftmost key was pushed
       // if (millis() > sprayer5lastOn + 20000L) {                     // if sprayer #5 has not been used for 20 seconds
          triggerK4(5, 1000, 1000, 1);                                // initiate spray on existing fountain
         // sprayer5lastOn = millis();                                  // remember when sprayer #5 was used
          lrKey = true;                                               // remember that a 'special' sprayer was used
        //}
      }
      if (i == MAX_INPUT_KEYS - 1) {                                  // if rightmost key was pushed
      //  if (millis() > sprayer6lastOn + 20000L) {                     // if sprayer #6 has not been used for 20 seconds
          triggerK4(6, 1000, 1000, 1);                                // initiate spray on roof
         // sprayer6lastOn = millis();                                  // remember when sprayer #6 was used
          lrKey = true;                                               // remember that a 'special' sprayer was used
      //  }
      }
      int wKey = whiteKeys[i];                                        // check if and which white key was pressed
      if (wKey >= 0 && !lrKey) {                                      // if white key was pushed and not outmost key
        triggerK4(wKey % 5, 400, 100, 1);                             // initiate spray for single white key
      }
      int bKey = blackKeys[i];                                        // check if and which black key was pressed
      if (bKey >= 0) {
        triggerK4(4 - (bKey % 5), 400, 100, 1);                       // initiate spray for single black key
      }
    }




    if (!sprayer[i] && pSprayer[i]) {                                 // if sprayer[] was just turned off
      int wKey = whiteKeys[i];                                        // check if and which white key was released
      if (wKey >= 0) {
        triggerK4(wKey % 5, 0, 0, 0);                                 // kill spray for single white key
      }
      int bKey = blackKeys[i];                                        // check if and which black key was pressed
      if (bKey >= 0) {
        triggerK4(4 - (bKey % 5), 0, 0, 0);                           // kill spray for single black key
      }
    }

    pSprayer[i] = sprayer[i];                                         // remember interpreter input state
  }                                                                   // end of cycling through all inputs

  /////////////////////////////////////////////////////////////////// LED interpreter
  if (keyPushed > -1) {                                             // if a key was pushed

    float f = keyPushed;
    f = f / (MAX_INPUT_KEYS - 1);

    float r1 = 255;
    float g1 = f * 150 + 15;
    float b1 = f * 50;

    alternateLED = !alternateLED;                                   // toggle which LED's should be addressed
    if (alternateLED) {
      fadeLEDs(0, 750,   int(r1), int(g1), int(b1),     int(r1) / 3, int(g1) / 5, int(b1) / 7);     // fade 12V LED's
    } else {
      fadeLEDs(1, 850,   int(r1), int(g1), int(b1),     int(r1) / 3, int(g1) / 5, int(b1) / 7);     // fade 12V LED's
    }
  }
  /////////////////////////////////////////////////////////////////// End of LED interpreter


  if (keyCount > 2) {                                                 // if 3 or more keys were pressed at the same time
    if (millis() > sprayer7lastOn + 5000) {                           // if sprayer #7 has not been used for 5 seconds
      triggerK4(7, 1000, 1000, 1);                                    // initiate spray top down in pool
      sprayer7lastOn = millis();                                      // remember when sprayer #7 was used
    }
  }


}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void interpreteCembalo() {
  /*  cembalo pressure sprayer interpretation:
       white keys will trigger sprayers in pool from left to right
       black keys will trigger sprayers in pool from right to left
       if there are more keys pushed than fountains are turned on in the pool:
       also spray down from top into pool
       and if one of the keys which are pushed while not enough fountains are turned on..
       ... is the leftmost key -> turn on spray on existing fountain (every 20s possible)
       ... is the rightmost key -> turn on spray on roof (every 20s possible)
  */
  int keyCount = 0;                                                   // used to count how many keys are pushed simultaneously (chord detection)
  for (int i = 0; i < MAX_INPUT_KEYS; i++) {                          // cycle through all notes
    sprayer[i] = synth[i];                                            // copy synth[]-array to sprayer[]-array
    keyCount += synth[i];                                             // count how many keys are pushed in order to detect chords
    if (sprayer[i] && !pSprayer[i]) {                                 // if sprayer[] was just turned on
      ledFlash = millis();                                            // remember time when a key was freshly pressed in order to calculate LED flash
      int wKey = whiteKeys[i];                                        // check if and which white key was pressed
      if (wKey >= 0) {                                                // if white key was pushed
        triggerK4(wKey % 5, 1500, 600, 1);            // initiate spray for single white key
        poolSprayer[wKey % 5] = true;
      }
      int bKey = blackKeys[i];                                        // check if and which black key was pressed
      if (bKey >= 0) {
        triggerK4(4 - (bKey % 5), 1500, 600, 1);      // initiate spray for single black key
        poolSprayer[4 - (bKey % 5)] = true;
      }
    }

    int activePoolSprayers = 0;                                       // count how many sprayers in the pool are currently turned on
    for (int i = 0; i < 5; i++) {
      activePoolSprayers += poolSprayer[i];
    }

    if (keyCount > activePoolSprayers) {                              // if more keys are pushed than sprayers are turned on:
      //                                                              /////////////////////// Special FX! ///////////////////
      triggerK4(7, 500, 1200, 1);                                       // initiate spray top down in pool
      sprayer7lastOn = millis();                                      // remember when sprayer #7 was used
      if (synth[0] == HIGH /*&& sprayer5lastOn + 20000L < millis()*/) {   // if leftmost key is pushed (every 20s):
        triggerK4(5, 500, 1500, 1);                                     // initiate spray on existing fountain
        sprayer5lastOn = millis();
      }
      if (synth[MAX_INPUT_KEYS - 1] == HIGH /*&& sprayer6lastOn + 20000L < millis()*/) { // if rightmost key is pushed (every 20s):
        triggerK4(6, 500, 500, 1);                                    // initiate spray on roof
        sprayer6lastOn = millis();
      }
      //                                                              /////////////////////// End of Special FX! ////////////
    }

    if (!sprayer[i] && pSprayer[i]) {                                 // if sprayer[] was just turned off
      int wKey = whiteKeys[i];                                        // check if and which white key was released
      if (wKey >= 0) {
        triggerK4(wKey % 5, 0, 0, 0);                                 // kill spray for single white key
      }
      int bKey = blackKeys[i];                                        // check if and which black key was pressed
      if (bKey >= 0) {
        triggerK4(4 - (bKey % 5), 0, 0, 0);                           // kill spray for single black key
      }
    }
    pSprayer[i] = sprayer[i];                                         // remember interpreter input state
  }                                                                   // end of cycling through all inputs

  if (keyCount == 0) {                                                // if no key is pressed
    for (int i = 0; i < 5; i++) {                                     // reset array that remembers which sprayers...
      poolSprayer[i] = 0;                                             // ...in the pool are turned on
    }
  } else {
    //////////////////////////////////////////////////////////////////// LED interpretation
    unsigned long r1 = millis(); // each primary color cycles through its own sinewave based on time
    r1 = r1 / 3;
    r1 = r1 % 256;
    unsigned long g1 = millis();
    g1 = g1 / 6;
    g1 = g1 % 256;
    unsigned long b1 = millis();
    b1 = b1 / 4;
    b1 = b1 % 256;

    unsigned long r2 = millis();
    r2 = r2 / 7;
    r2 = r2 % 256;
    unsigned long g2 = millis();
    g2 = g2 / 5;
    g2 = g2 % 256;
    unsigned long b2 = millis();
    b2 = b2 / 8;
    b2 = b2 % 256;

    unsigned long fade = millis() - ledFlash; // add a flash when key is freshly pressed and fade out when keys remain pressed for long time
    fade = constrain(fade, 0, 4000);
    fade = 4000 - fade;
    float fFade = fade;
    fFade = fFade / 3000;

    float flash = (fFade - 1.2);
    if (flash < 0) {
      flash = 0;
    }
    flash = flash * 1000;

    fadeLEDs(0, 500,   constrain(fFade * sine[r1] + flash, 0, 255), constrain(.7 * fFade * sine[g1] + flash, 0, 255), constrain(fFade * sine[b1] + flash, 0, 255), sine[r2] / 8, sine[g2] / 8, sine[b2] / 8); // fade 12V LED's
    fadeLEDs(1, 500,   constrain(fFade * sine[r2] + flash, 0, 255), constrain(.7 * fFade * sine[g2] + flash, 0, 255), constrain(fFade * sine[b2] + flash, 0, 255), sine[r1] / 8, sine[g1] / 8, sine[b1] / 8); // fade 12V LED's
  }
}
