// functions that care about opening old log files, playing content from them, closing logfiles

boolean startPlayBack() {
  recFileName = createFilename(random(0, fileNumber));         // create a filename

  Serial.println("---------------------------------------");
  Serial.print("Play back: ");
  Serial.println(recFileName);
  oldLogFile = SD.open(recFileName);                                  // open old logfile
  recLineCount = 0;                                                   // reset line count
  recTimebaseStart = millis();                                        // remember the time when playback started
  recNoteAvailable = false;                                           // no note available yet
  return boolean(oldLogFile);
}

void closePlayBack() {
  if (VERBOSE_PLAYBACK)Serial.println(">>>closing playback file");
  oldLogFile.close();
  for (int i = 0; i < MAX_INPUT_KEYS; i++) { // go through all keys and release them
    synth[i] = 0;
  }
  playSynth();  //release keys
  playBackMode = false;
}

//

void parseLine() {
  if (VERBOSE_PLAYBACK)Serial.print("/parse line #");
  if (VERBOSE_PLAYBACK)Serial.print(recLineCount);
  if (VERBOSE_PLAYBACK)Serial.print(" in File ");
  if (VERBOSE_PLAYBACK)Serial.print(recFileName);
  if (VERBOSE_PLAYBACK)Serial.print(" : ");
  String dataLine = "";                                             // reset dataLine
  boolean endOfLine = false;                                        // reset end of line flag
  while (oldLogFile.available() && endOfLine == false) { //----------- parse through incoming bytes until end of line is reached
    char character = oldLogFile.read();                             // read one byte time
    dataLine += character;                                          // assemble a whole line of incoming data
    if (character == 10) {                    //         .......... if end of line is reached:
      endOfLine = true;                                             // flag end of line

      if (recLineCount == 0) {                                      // read file header
        Serial.println("HEADER:");
        Serial.print(dataLine);
        String fileHour = dataLine.substring(8, 12);                // get rough area of hour in file
        String playBackHour = "";
        for (int i = 0; i < 4; i++) {                               // parse in order to get exact hour
          String singleChar = fileHour.substring(i, i + 1);
          if (singleChar.equals(":")) {
            break;
          }
          if (!singleChar.equals(" ")) {
            playBackHour = playBackHour + singleChar;
          }
        }
        int fileTime = playBackHour.toInt();
        Serial.print("extracted hour: ");
        Serial.println(fileTime);
        if (fileTime > 21 || fileTime < 8) {                         // if recording happened when installation was turned off
          Serial.println("offTime detected");
          playBackMode = false;                                      // jump out of playback mode
        }

      }

      if (recLineCount > 1) { // .................................. if we are below header: parse line content
        recNote = dataLine.substring(0, 2).toInt();                 // get note value
        recNoteOn = dataLine.substring(5, 7).equals("ON");          // get on/off value
        instrument = dataLine.substring(10, 13).toInt();             // get instrument
        recNoteTime = dataLine.substring(14).toInt();               // get time
        recNoteAvailable = true;                                    // set flag - note is available
        if (recLineCount == 2) { //              ...................if this is the first note in the file:
          recTimebaseRecording = recNoteTime;                       // set timebase for data in file
        }
      }
      recLineCount++;                                               // increment line count
      if (VERBOSE_PLAYBACK)Serial.print(dataLine);
      dataLine = "";                                                // reset line parsing string
    }
  }
}


//uses values recNote, recNoteOn, recNoteTime, and plays note when it's time

void pushBufferToSynth() {
  unsigned long playTime = recNoteTime - recTimebaseRecording + recTimebaseStart; // calculate when note should be played
  if (recNoteAvailable && millis() > playTime) {              // if there's a note available and if it is time to play it
    synth[recNote] = recNoteOn;                               // copy note to synth[]
    recNoteAvailable = false;                                 // reset note availability flag
  }
}
